/*  pcm2wav v1.0  Copyright (C) 1996  Byron Stanoszek
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **************************************************************************/

/* pcm2wav.c */

#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>

#define GRDL_VER "v1.0"


/* writes 'num' to file using len number of bytes */
void putref(f, num, len)
FILE *f;
int num, len;
{
  unsigned int a=num;
  int b;

  for(b=0;b<len;b++)
    fputc((a >> b*8)&255, f);
}

/* reads len number of bytes from file and turns them into integer format */
int getref(f, len)
FILE *f;
int len;
{
  unsigned int a=0, b;

  for(b=0;b < len;b++)
    a=(a<<8)+getc(f);
  return a;
}

void main(argc, argv)
int argc;
char *argv[];
{
  FILE *f, *g;
  char *cmd=argv[0];
  int a, c;
  int loop, len, typ;

  /* grab any flags on the command line */
  while(argc > 1) {
    --argc; ++argv;
    if(**argv == '-') {
      for(a=1;(*argv)[a];a++) {
        switch((*argv)[a]) {
        case 'v':
          fprintf(stdout, "%s: GRDL Version Reference %s\n", cmd, GRDL_VER);
          fflush(stdout);
          exit(0);
        default:
          fprintf(stderr, "%s: illegal option -- %c\n", cmd, (*argv)[a]);
          fprintf(stderr, "Usage: %s [-v] file.pcm file.wav\n", cmd);
          fflush(stderr);
          exit(1);
        }
      }
      continue;
    } else
      break;
  }

  /* open .pcm file for reading */
  if(!(f=fopen(*argv, "r"))) {
    perror(*argv);
    exit(1);
  }

  /* read file header */
  typ=getref(f, 2);

  if(typ != 2) {
    fprintf(stderr, "%s: File not PCM format.\n", *argv);
    exit(1);
  }

  /* open destination .wav file for writing */
  if(!(g=fopen(*(++argv), "w"))) {
    perror(*argv);
    exit(1);
  }

  loop=getref(f, 4);
  len=getref(f, 4);

  /* write file header */
  fprintf(g, "RIFF");
  putref(g, len+36, 4); /* file length */
  fprintf(g, "WAVEfmt ");
  putref(g, 16, 4); /* Length of Subchunk; always 16 bytes */
  putref(g, 1, 2); /* PCM Format = 1 */
  putref(g, 1, 2); /* 1=Mono, 2=Stereo */
  putref(g, 16000, 4); /* Sample Frequency */
  putref(g, 16000, 4); /* bytes/sec */
  putref(g, 1, 2); /* bytes per sample */
  putref(g, 8, 2); /* bits per sample */
  fprintf(g, "data");
  putref(g, len, 4); /* length of data */

  /* start of music */
  fseek(f, 2048, SEEK_SET);

  while((c=fgetc(f)) != EOF) {
    c=fgetc(f);
    if(c > 127)
      c=127-c;
    else
      c--;
    fputc(c+128, g);
  }

  fclose(f);
  fclose(g);

  printf("Converted %d samples to WAV.\n", len);
  fflush(stdout);
  exit(0);
}
